﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace CIntersectingTetrahedrons
{
    public partial class TetrahedronForm : Form
    {
        public TetrahedronForm()
        {
            InitializeComponent();
        }

        // The Direct3D device.
        private Device m_Device;

        // Data variables.
        private const int NUM_TRIANGLES = 8;
        private const int NUM_POINTS = 3 * NUM_TRIANGLES;

        // The vertex buffer that holds drawing data.
        private VertexBuffer m_VertexBuffer = null;

        // Initialize the graphics device. Return true if successful.
        public bool InitializeGraphics()
        {
            try
            {
                PresentParameters parameters = new PresentParameters();
                parameters.Windowed = true;
                parameters.SwapEffect = SwapEffect.Discard;
                parameters.EnableAutoDepthStencil = true;            // Depth stencil on.
                parameters.AutoDepthStencilFormat = DepthFormat.D16;

                // Use hardware vertex processing if supported.
                m_Device = new Device(0, DeviceType.Hardware, this,
                    CreateFlags.SoftwareVertexProcessing, parameters);

                // Don't rely on culling to remove hidden surfaces.
                m_Device.RenderState.CullMode = Cull.None;
                m_Device.RenderState.FillMode = FillMode.Solid;

                // Turn on the Z-buffer and disable lighting.
                m_Device.RenderState.ZBufferEnable = true;
                m_Device.RenderState.Lighting = false;

                // Create the vertex data.
                CreateVertexBuffer();

                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error initializing Direct3D\n\n" + ex.Message,
                    "Direct3D Error", MessageBoxButtons.OK);
                return false;
            }
        }

        public void Render()
        {
            if (m_Device == null) return;

            // Clear the back buffer.
            m_Device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, this.BackColor, 1, 0);

            // Make a scene.
            m_Device.BeginScene();

            // Setup the world, view, and projection matrices.
            SetupMatrices();

            // Set the device's data stream source (the vertex buffer).
            m_Device.SetStreamSource(0, m_VertexBuffer, 0);

            // Tell the device the format of the vertices.
            m_Device.VertexFormat = CustomVertex.PositionColored.Format;

            // Draw the primitives in the data stream.
            m_Device.DrawPrimitives(PrimitiveType.TriangleList, 0, NUM_TRIANGLES);

            // End the scene and display.
            m_Device.EndScene();
            m_Device.Present();
        }

        // Create a vertex buffer for the device.
        public void CreateVertexBuffer()
        {
            // Create a vertex buffer.
            m_VertexBuffer = new VertexBuffer(
                typeof(CustomVertex.PositionColored),
                NUM_POINTS, m_Device, 0,
                CustomVertex.PositionColored.Format,
                Pool.Default);

            // Lock the vertex buffer. 
            // Lock returns an array of positionColored objects.
            CustomVertex.PositionColored[] vertices =
                (CustomVertex.PositionColored[])m_VertexBuffer.Lock(0, 0);
            int start_index = 0;

            float sqrt3 = (float)Math.Sqrt(3f);
            float y = (float)(2f * Math.Sqrt(2f / 3f));
            float offset = y / 3f;
            MakeTriangle(vertices, ref start_index, Color.FromArgb(255, 0, 128, 0),
                0, 0 - offset, sqrt3 - 1 / sqrt3,
                0, y - offset, 0,
                -1, 0 - offset, -1 / sqrt3);
            MakeTriangle(vertices, ref start_index, Color.FromArgb(255, 0, 108, 0),
                0, 0 - offset, sqrt3 - 1 / sqrt3,
                1, 0 - offset, -1 / sqrt3,
                0, y - offset, 0);
            MakeTriangle(vertices, ref start_index, Color.FromArgb(255, 0, 88, 0),
                1, 0 - offset, -1 / sqrt3,
                -1, 0 - offset, -1 / sqrt3,
                0, y - offset, 0);
            MakeTriangle(vertices, ref start_index, Color.DarkGreen,
                0, 0 - offset, sqrt3 - 1 / sqrt3,
                -1, 0 - offset, -1 / sqrt3,
                1, 0 - offset, -1 / sqrt3);

            MakeTriangle(vertices, ref start_index, Color.FromArgb(255, 0, 0, 128),
                0, -(0 - offset), sqrt3 - 1 / sqrt3,
                0, -(y - offset), 0,
                -1, -(0 - offset), -1 / sqrt3);
            MakeTriangle(vertices, ref start_index, Color.FromArgb(255, 0, 0, 108),
                0, -(0 - offset), sqrt3 - 1 / sqrt3,
                1, -(0 - offset), -1 / sqrt3,
                0, -(y - offset), 0);
            MakeTriangle(vertices, ref start_index, Color.FromArgb(255, 0, 0, 88),
                1, -(0 - offset), -1 / sqrt3,
                -1, -(0 - offset), -1 / sqrt3,
                0, -(y - offset), 0);
            MakeTriangle(vertices, ref start_index, Color.DarkBlue,
                0, -(0 - offset), sqrt3 - 1 / sqrt3,
                -1, -(0 - offset), -1 / sqrt3,
                1, -(0 - offset), -1 / sqrt3);

            m_VertexBuffer.Unlock();
        }

        private void MakeTriangle(CustomVertex.PositionColored[] vertices,
            ref int start_index, Color clr,
            float x1, float y1, float z1,
            float x2, float y2, float z2,
            float x3, float y3, float z3)
        {
            const float PT_SCALE = 2;
            x1 *= PT_SCALE;
            y1 *= PT_SCALE;
            z1 *= PT_SCALE;
            x2 *= PT_SCALE;
            y2 *= PT_SCALE;
            z2 *= PT_SCALE;
            x3 *= PT_SCALE;
            y3 *= PT_SCALE;
            z3 *= PT_SCALE;

            // It's already oriented clockwise.
            vertices[start_index].X = x1;
            vertices[start_index].Y = y1;
            vertices[start_index].Z = z1;
            vertices[start_index].Color = clr.ToArgb();
            start_index++;

            vertices[start_index].X = x2;
            vertices[start_index].Y = y2;
            vertices[start_index].Z = z2;
            vertices[start_index].Color = clr.ToArgb();
            start_index++;

            vertices[start_index].X = x3;
            vertices[start_index].Y = y3;
            vertices[start_index].Z = z3;
            vertices[start_index].Color = clr.ToArgb();
            start_index++;
        }

        // Setup the world, view, and projection matrices.
        private void SetupMatrices()
        {
            // World Matrix:
            // Rotate the object around a crooked axis by
            // 2 * Pi radians per 3000 ticks (3 seconds).
            const int TICKS_PER_REV = 3000;
            int ms_rotated = Environment.TickCount % TICKS_PER_REV;
            float angle = (float)(ms_rotated * (2 * Math.PI) / TICKS_PER_REV);
            m_Device.Transform.World = Matrix.RotationYawPitchRoll((float)Math.PI / 2, (float)Math.PI / 4, angle);

            // View Matrix:
            // This is defined by giving:
            //       An eye point            (0, 3, -10)
            //       A point to look at      (0, 0, 0)
            //       An "up" direction       <0, 1, 0>
            m_Device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 3, -10),
                new Vector3(0, 0, 0),
                new Vector3(0, 1, 0));

            // Projection Matrix:
            // Perspective transformation defined by:
            //       Field of view           Pi / 4
            //       Aspect ratio            1
            //       Near clipping plane     Z = 1
            //       Far clipping plane      Z = 10
            m_Device.Transform.Projection =
                Matrix.PerspectiveFovLH((float)(Math.PI / 4f), 1, 1, 100);
        }       
    }
}
