﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace COctahedron
{
    public partial class OctahedronForm : Form
    {
        public OctahedronForm()
        {
            InitializeComponent();
        }

        // The Direct3D device.
        private Device m_Device;

        // Data variables.
        private const int NUM_TRIANGLES = 8;
        private const int NUM_POINTS = 3 * NUM_TRIANGLES;

        // The vertex buffer that holds drawing data.
        private VertexBuffer m_VertexBuffer = null;

        // Initialize the graphics device. Return true if successful.
        public bool InitializeGraphics()
        {
            try
            {
                PresentParameters parameters = new PresentParameters();
                parameters.Windowed = true;
                parameters.SwapEffect = SwapEffect.Discard;

                // Use hardware vertex processing if supported.
                m_Device = new Device(0, DeviceType.Hardware, this,
                    CreateFlags.SoftwareVertexProcessing, parameters);

                // Turn off D3D lighting because 
                // we set the vertex colors explicitly.
                m_Device.RenderState.Lighting = false;

                // Create the vertex data.
                CreateVertexBuffer();

                return true;
            }
            catch(Exception ex)
            {
                MessageBox.Show("Error initializing Direct3D\n\n" + ex.Message,
                    "Direct3D Error", MessageBoxButtons.OK);
                return false;
            }
        }

        public void Render()
        {
            if (m_Device == null) return;

            // Clear the back buffer.
            m_Device.Clear(ClearFlags.Target, this.BackColor, 1, 0);

            // Make a scene.
            m_Device.BeginScene();

            // Setup the world, view, and projection matrices.
            SetupMatrices();

            // Set the device's data stream source (the vertex buffer).
            m_Device.SetStreamSource(0, m_VertexBuffer, 0);

            // Tell the device the format of the vertices.
            m_Device.VertexFormat = CustomVertex.PositionColored.Format;

            // Draw the primitives in the data stream.
            m_Device.DrawPrimitives(PrimitiveType.TriangleList, 0, NUM_TRIANGLES);

            // End the scene and display.
            m_Device.EndScene();
            m_Device.Present();
        }

        // Create a vertex buffer for the device.
        public void CreateVertexBuffer()
        {
            // Create a vertex buffer.
            m_VertexBuffer = new VertexBuffer(
                typeof(CustomVertex.PositionColored),
                NUM_POINTS, m_Device, 0,
                CustomVertex.PositionColored.Format,
                Pool.Default);

            // Lock the vertex buffer. 
            // Lock returns an array of positionColored objects.
            CustomVertex.PositionColored[] vertices =
                (CustomVertex.PositionColored[])m_VertexBuffer.Lock(0, 0);
            int start_index = 0;

            const float WID = 2;
            MakeTriangle(vertices, ref start_index, Color.Red,
                0, WID, 0,
                0, 0, WID,
                WID, 0, 0);
            MakeTriangle(vertices, ref start_index, Color.Yellow,
                0, WID, 0,
                WID, 0, 0,
                0, 0, -WID);
            MakeTriangle(vertices, ref start_index, Color.Red,
                0, WID, 0,
                0, 0, -WID,
                -WID, 0, 0);
            MakeTriangle(vertices, ref start_index, Color.Yellow,
                0, WID, 0,
                -WID, 0, 0,
                0, 0, WID);
            MakeTriangle(vertices, ref start_index, Color.Blue,
                0, -WID, 0,
                WID, 0, 0,
                0, 0, WID);
            MakeTriangle(vertices, ref start_index, Color.Green,
                0, -WID, 0,
                0, 0, -WID,
                WID, 0, 0);
            MakeTriangle(vertices, ref start_index, Color.Blue,
                0, -WID, 0,
                -WID, 0, 0,
                0, 0, -WID);
            MakeTriangle(vertices, ref start_index, Color.Green,
                0, -WID, 0,
                0, 0, WID,
                -WID, 0, 0);

            m_VertexBuffer.Unlock();
        }

        private void MakeTriangle(CustomVertex.PositionColored[] vertices,
            ref int start_index, Color clr,
            float x1, float y1, float z1,
            float x2, float y2, float z2,
            float x3, float y3, float z3)
        {
            vertices[start_index].X = x1;
            vertices[start_index].Y = y1;
            vertices[start_index].Z = z1;
            vertices[start_index].Color = clr.ToArgb();
            start_index++;

            vertices[start_index].X = x2;
            vertices[start_index].Y = y2;
            vertices[start_index].Z = z2;
            vertices[start_index].Color = clr.ToArgb();
            start_index++;

            vertices[start_index].X = x3;
            vertices[start_index].Y = y3;
            vertices[start_index].Z = z3;
            vertices[start_index].Color = clr.ToArgb();
            start_index++;
        }

        // Setup the world, view, and projection matrices.
        private void SetupMatrices()
        {
            // World Matrix:
            // Rotate the object around the Y axis by
            // 2 * Pi radians per 2000 ticks (2 seconds).
            const int TICKS_PER_REV = 2000;
            int ms_rotated = Environment.TickCount % TICKS_PER_REV;
            float angle = (float)(ms_rotated * (2 * Math.PI) / TICKS_PER_REV);
            m_Device.Transform.World = Matrix.RotationY(angle);

            // View Matrix:
            // This is defined by giving:
            //       An eye point            (0, 3, -10)
            //       A point to look at      (0, 0, 0)
            //       An "up" direction       <0, 1, 0>
            m_Device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 3, -10),
                new Vector3(0, 0, 0),
                new Vector3(0, 1, 0));

            // Projection Matrix:
            // Perspective transformation defined by:
            //       Field of view           Pi / 4
            //       Aspect ratio            1
            //       Near clipping plane     Z = 1
            //       Far clipping plane      Z = 100
            m_Device.Transform.Projection =
                Matrix.PerspectiveFovLH((float)(Math.PI / 4), 1, 1, 100);
        }
    }
}
