﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace CSphere
{
    public partial class SphereForm : Form
    {
        public SphereForm()
        {
            InitializeComponent();
        }

        // The Direct3D device.
        private Device m_Device;

        // Data variables.
        private const int NUM_TRIANGLES = 80;
        private const int NUM_POINTS = 3 * NUM_TRIANGLES;

        // The vertex buffer that holds drawing data.
        private VertexBuffer m_VertexBuffer = null;

        // Initialize the graphics device. Return true if successful.
        public bool InitializeGraphics()
        {
            try
            {
                PresentParameters parameters = new PresentParameters();
                parameters.Windowed = true;
                parameters.SwapEffect = SwapEffect.Discard;
                parameters.EnableAutoDepthStencil = true;            // Depth stencil on.
                parameters.AutoDepthStencilFormat = DepthFormat.D16;

                // Use hardware vertex processing if supported.
                m_Device = new Device(0, DeviceType.Hardware, this,
                    CreateFlags.SoftwareVertexProcessing, parameters);

                // Turn on the Z-buffer and enable lighting.
                m_Device.RenderState.ZBufferEnable = true;
                m_Device.RenderState.Lighting = true;

                // Cull triangles that are oriented counter clockwise.
                m_Device.RenderState.CullMode = Cull.CounterClockwise;

                // Create the vertex data.
                CreateVertexBuffer();

                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error initializing Direct3D\n\n" + ex.Message,
                    "Direct3D Error", MessageBoxButtons.OK);
                return false;
            }
        }

        public void Render()
        {
            if (m_Device == null) return;

            // Clear the back buffer.
            m_Device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, this.BackColor, 1, 0);

            // Make a scene.
            m_Device.BeginScene();

            // Setup the world, view, and projection matrices.
            SetupMatrices();

            // Make the material.
            SetupMaterial();

            // Make the lights.
            SetupLights();

            // Set the device's data stream source (the vertex buffer).
            m_Device.SetStreamSource(0, m_VertexBuffer, 0);

            // Tell the device the format of the vertices.
            m_Device.VertexFormat = CustomVertex.PositionNormal.Format;

            // Draw the primitives in the data stream.
            m_Device.DrawPrimitives(PrimitiveType.TriangleList, 0, NUM_TRIANGLES);

            // End the scene and display.
            m_Device.EndScene();
            m_Device.Present();
        }

        // Create a vertex buffer for the device.
        public void CreateVertexBuffer()
        {
            // Create a vertex buffer.
            m_VertexBuffer = new VertexBuffer(
                typeof(CustomVertex.PositionNormal),
                NUM_POINTS, m_Device, 0,
                CustomVertex.PositionNormal.Format,
                Pool.Default);

            // Lock the vertex buffer. 
            // Lock returns an array of PositionNormal objects.
            CustomVertex.PositionNormal[] vertices =
                (CustomVertex.PositionNormal[])m_VertexBuffer.Lock(0, 0);
            int start_index = 0;

            MakeTriangle(vertices, ref start_index,
                0, 0, 95.10565,
                47.55283, 15.45085, 80.9017,
                0, 50, 80.9017);
            MakeTriangle(vertices, ref start_index,
                47.55283, 15.45085, 80.9017,
                47.55283, 65.45085, 49.99999,
                0, 50, 80.9017);
            MakeTriangle(vertices, ref start_index,
                47.55283, 15.45085, 80.9017,
                80.9017, 26.28656, 42.53253,
                47.55283, 65.45085, 49.99999);
            MakeTriangle(vertices, ref start_index,
                0, 50, 80.9017,
                47.55283, 65.45085, 49.99999,
                0, 85.06508, 42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, 95.10565,
                0, 50, 80.9017,
                -47.55283, 15.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                0, 50, 80.9017,
                -47.55283, 65.45085, 49.99999,
                -47.55283, 15.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                0, 50, 80.9017,
                0, 85.06508, 42.53253,
                -47.55283, 65.45085, 49.99999);
            MakeTriangle(vertices, ref start_index,
                -47.55283, 15.45085, 80.9017,
                -47.55283, 65.45085, 49.99999,
                -80.9017, 26.28656, 42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, 95.10565,
                -47.55283, 15.45085, 80.9017,
                -29.38927, -40.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                -47.55283, 15.45085, 80.9017,
                -76.94209, -25, 49.99999,
                -29.38927, -40.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                -47.55283, 15.45085, 80.9017,
                -80.9017, 26.28656, 42.53253,
                -76.94209, -25, 49.99999);
            MakeTriangle(vertices, ref start_index,
                -29.38927, -40.45085, 80.9017,
                -76.94209, -25, 49.99999,
                -50, -68.81909, 42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, 95.10565,
                -29.38927, -40.45085, 80.9017,
                29.38927, -40.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                -29.38927, -40.45085, 80.9017,
                0, -80.9017, 50,
                29.38927, -40.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                -29.38927, -40.45085, 80.9017,
                -50, -68.81909, 42.53253,
                0, -80.9017, 50);
            MakeTriangle(vertices, ref start_index,
                29.38927, -40.45085, 80.9017,
                0, -80.9017, 50,
                50, -68.81909, 42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, 95.10565,
                29.38927, -40.45085, 80.9017,
                47.55283, 15.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                29.38927, -40.45085, 80.9017,
                76.94209, -25, 49.99999,
                47.55283, 15.45085, 80.9017);
            MakeTriangle(vertices, ref start_index,
                29.38927, -40.45085, 80.9017,
                50, -68.81909, 42.53253,
                76.94209, -25, 49.99999);
            MakeTriangle(vertices, ref start_index,
                47.55283, 15.45085, 80.9017,
                76.94209, -25, 49.99999,
                80.9017, 26.28656, 42.53253);
            MakeTriangle(vertices, ref start_index,
                80.9017, 26.28656, 42.53253,
                76.94209, -25, 49.99999,
                95.10565, 0, 0);
            MakeTriangle(vertices, ref start_index,
                76.94209, -25, 49.99999,
                76.94209, -55.9017, 0,
                95.10565, 0, 0);
            MakeTriangle(vertices, ref start_index,
                76.94209, -25, 49.99999,
                50, -68.81909, 42.53253,
                76.94209, -55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                95.10565, 0, 0,
                76.94209, -55.9017, 0,
                80.9017, -26.28656, -42.53253);
            MakeTriangle(vertices, ref start_index,
                80.9017, 26.28656, 42.53253,
                95.10565, 0, 0,
                76.94209, 55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                95.10565, 0, 0,
                76.94209, 25, -49.99999,
                76.94209, 55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                95.10565, 0, 0,
                80.9017, -26.28656, -42.53253,
                76.94209, 25, -49.99999);
            MakeTriangle(vertices, ref start_index,
                76.94209, 55.9017, 0,
                76.94209, 25, -49.99999,
                50, 68.81909, -42.53253);
            MakeTriangle(vertices, ref start_index,
                80.9017, 26.28656, 42.53253,
                76.94209, 55.9017, 0,
                47.55283, 65.45085, 49.99999);
            MakeTriangle(vertices, ref start_index,
                76.94209, 55.9017, 0,
                29.38927, 90.45085, 0,
                47.55283, 65.45085, 49.99999);
            MakeTriangle(vertices, ref start_index,
                76.94209, 55.9017, 0,
                50, 68.81909, -42.53253,
                29.38927, 90.45085, 0);
            MakeTriangle(vertices, ref start_index,
                47.55283, 65.45085, 49.99999,
                29.38927, 90.45085, 0,
                0, 85.06508, 42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 85.06508, 42.53253,
                29.38927, 90.45085, 0,
                -29.38927, 90.45085, 0);
            MakeTriangle(vertices, ref start_index,
                29.38927, 90.45085, 0,
                0, 80.9017, -50,
                -29.38927, 90.45085, 0);
            MakeTriangle(vertices, ref start_index,
                29.38927, 90.45085, 0,
                50, 68.81909, -42.53253,
                0, 80.9017, -50);
            MakeTriangle(vertices, ref start_index,
                -29.38927, 90.45085, 0,
                0, 80.9017, -50,
                -50, 68.81909, -42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 85.06508, 42.53253,
                -29.38927, 90.45085, 0,
                -47.55283, 65.45085, 49.99999);
            MakeTriangle(vertices, ref start_index,
                -29.38927, 90.45085, 0,
                -76.94209, 55.9017, 0,
                -47.55283, 65.45085, 49.99999);
            MakeTriangle(vertices, ref start_index,
                -29.38927, 90.45085, 0,
                -50, 68.81909, -42.53253,
                -76.94209, 55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                -47.55283, 65.45085, 49.99999,
                -76.94209, 55.9017, 0,
                -80.9017, 26.28656, 42.53253);
            MakeTriangle(vertices, ref start_index,
                -80.9017, 26.28656, 42.53253,
                -76.94209, 55.9017, 0,
                -95.10565, 0, 0);
            MakeTriangle(vertices, ref start_index,
                -76.94209, 55.9017, 0,
                -76.94209, 25, -49.99999,
                -95.10565, 0, 0);
            MakeTriangle(vertices, ref start_index,
                -76.94209, 55.9017, 0,
                -50, 68.81909, -42.53253,
                -76.94209, 25, -49.99999);
            MakeTriangle(vertices, ref start_index,
                -95.10565, 0, 0,
                -76.94209, 25, -49.99999,
                -80.9017, -26.28656, -42.53253);
            MakeTriangle(vertices, ref start_index,
                -80.9017, 26.28656, 42.53253,
                -95.10565, 0, 0,
                -76.94209, -25, 49.99999);
            MakeTriangle(vertices, ref start_index,
                -95.10565, 0, 0,
                -76.94209, -55.9017, 0,
                -76.94209, -25, 49.99999);
            MakeTriangle(vertices, ref start_index,
                -95.10565, 0, 0,
                -80.9017, -26.28656, -42.53253,
                -76.94209, -55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                -76.94209, -25, 49.99999,
                -76.94209, -55.9017, 0,
                -50, -68.81909, 42.53253);
            MakeTriangle(vertices, ref start_index,
                -50, -68.81909, 42.53253,
                -76.94209, -55.9017, 0,
                -29.38927, -90.45085, 0);
            MakeTriangle(vertices, ref start_index,
                -76.94209, -55.9017, 0,
                -47.55283, -65.45085, -49.99999,
                -29.38927, -90.45085, 0);
            MakeTriangle(vertices, ref start_index,
                -76.94209, -55.9017, 0,
                -80.9017, -26.28656, -42.53253,
                -47.55283, -65.45085, -49.99999);
            MakeTriangle(vertices, ref start_index,
                -29.38927, -90.45085, 0,
                -47.55283, -65.45085, -49.99999,
                0, -85.06508, -42.53253);
            MakeTriangle(vertices, ref start_index,
                -50, -68.81909, 42.53253,
                -29.38927, -90.45085, 0,
                0, -80.9017, 50);
            MakeTriangle(vertices, ref start_index,
                -29.38927, -90.45085, 0,
                29.38927, -90.45085, 0,
                0, -80.9017, 50);
            MakeTriangle(vertices, ref start_index,
                -29.38927, -90.45085, 0,
                0, -85.06508, -42.53253,
                29.38927, -90.45085, 0);
            MakeTriangle(vertices, ref start_index,
                0, -80.9017, 50,
                29.38927, -90.45085, 0,
                50, -68.81909, 42.53253);
            MakeTriangle(vertices, ref start_index,
                50, -68.81909, 42.53253,
                29.38927, -90.45085, 0,
                76.94209, -55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                29.38927, -90.45085, 0,
                47.55283, -65.45085, -49.99999,
                76.94209, -55.9017, 0);
            MakeTriangle(vertices, ref start_index,
                29.38927, -90.45085, 0,
                0, -85.06508, -42.53253,
                47.55283, -65.45085, -49.99999);
            MakeTriangle(vertices, ref start_index,
                76.94209, -55.9017, 0,
                47.55283, -65.45085, -49.99999,
                80.9017, -26.28656, -42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, -95.10565,
                29.38927, 40.45085, -80.9017,
                47.55283, -15.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                29.38927, 40.45085, -80.9017,
                76.94209, 25, -49.99999,
                47.55283, -15.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                29.38927, 40.45085, -80.9017,
                50, 68.81909, -42.53253,
                76.94209, 25, -49.99999);
            MakeTriangle(vertices, ref start_index,
                47.55283, -15.45085, -80.9017,
                76.94209, 25, -49.99999,
                80.9017, -26.28656, -42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, -95.10565,
                47.55283, -15.45085, -80.9017,
                0, -50, -80.9017);
            MakeTriangle(vertices, ref start_index,
                47.55283, -15.45085, -80.9017,
                47.55283, -65.45085, -49.99999,
                0, -50, -80.9017);
            MakeTriangle(vertices, ref start_index,
                47.55283, -15.45085, -80.9017,
                80.9017, -26.28656, -42.53253,
                47.55283, -65.45085, -49.99999);
            MakeTriangle(vertices, ref start_index,
                0, -50, -80.9017,
                47.55283, -65.45085, -49.99999,
                0, -85.06508, -42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, -95.10565,
                0, -50, -80.9017,
                -47.55283, -15.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                0, -50, -80.9017,
                -47.55283, -65.45085, -49.99999,
                -47.55283, -15.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                0, -50, -80.9017,
                0, -85.06508, -42.53253,
                -47.55283, -65.45085, -49.99999);
            MakeTriangle(vertices, ref start_index,
                -47.55283, -15.45085, -80.9017,
                -47.55283, -65.45085, -49.99999,
                -80.9017, -26.28656, -42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, -95.10565,
                -47.55283, -15.45085, -80.9017,
                -29.38927, 40.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                -47.55283, -15.45085, -80.9017,
                -76.94209, 25, -49.99999,
                -29.38927, 40.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                -47.55283, -15.45085, -80.9017,
                -80.9017, -26.28656, -42.53253,
                -76.94209, 25, -49.99999);
            MakeTriangle(vertices, ref start_index,
                -29.38927, 40.45085, -80.9017,
                -76.94209, 25, -49.99999,
                -50, 68.81909, -42.53253);
            MakeTriangle(vertices, ref start_index,
                0, 0, -95.10565,
                -29.38927, 40.45085, -80.9017,
                29.38927, 40.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                -29.38927, 40.45085, -80.9017,
                0, 80.9017, -50,
                29.38927, 40.45085, -80.9017);
            MakeTriangle(vertices, ref start_index,
                -29.38927, 40.45085, -80.9017,
                -50, 68.81909, -42.53253,
                0, 80.9017, -50);
            MakeTriangle(vertices, ref start_index,
                29.38927, 40.45085, -80.9017,
                0, 80.9017, -50,
                50, 68.81909, -42.53253);

            m_VertexBuffer.Unlock();
        }

        private void MakeTriangle(
            CustomVertex.PositionNormal[] vertices,
            ref int start_index,
            double x1, double y1, double z1,
            double x2, double y2, double z2,
            double x3, double y3, double z3)
        {
            const double PT_SCALE = 1f / 30f;
            x1 *= PT_SCALE;
            y1 *= PT_SCALE;
            z1 *= PT_SCALE;
            x2 *= PT_SCALE;
            y2 *= PT_SCALE;
            z2 *= PT_SCALE;
            x3 *= PT_SCALE;
            y3 *= PT_SCALE;
            z3 *= PT_SCALE;

            // Find a normal vector.
            Vector3 normal = CrossProduct(
                (float)(x2 - x1), (float)(y2 - y1), (float)(z2 - z1),
                (float)(x3 - x2), (float)(y3 - y2), (float)(z3 - z2));

            vertices[start_index].X = (float)x1;
            vertices[start_index].Y = (float)y1;
            vertices[start_index].Z = (float)z1;
            vertices[start_index].Normal = normal;
            start_index++;

            vertices[start_index].X = (float)x2;
            vertices[start_index].Y = (float)y2;
            vertices[start_index].Z = (float)z2;
            vertices[start_index].Normal = normal;
            start_index++;

            vertices[start_index].X = (float)x3;
            vertices[start_index].Y = (float)y3;
            vertices[start_index].Z = (float)z3;
            vertices[start_index].Normal = normal;
            start_index++;
        }

        // Return the cross product of two vectors.
        private Vector3 CrossProduct(
            float x1, float y1, float z1,
            float x2, float y2, float z2)
        {
            Vector3 v1 = new Vector3(x1, y1, z1);
            Vector3 v2 = new Vector3(x2, y2, z2);
            return Vector3.Cross(v1, v2);
        }

        // Setup the world, view, and projection matrices.
        private void SetupMatrices()
        {
            // World Matrix:
            // Rotate the object around a crooked axis by
            // 2 * Pi radians per 4000 ticks (4 seconds).
            const int TICKS_PER_REV = 4000;
            int ms_rotated = Environment.TickCount % TICKS_PER_REV;
            float angle = (float)(ms_rotated * (2 * Math.PI) / TICKS_PER_REV);
            m_Device.Transform.World = Matrix.RotationY(angle);

            // View Matrix:
            // This is defined by giving:
            //       An eye point            (0, 3, -10)
            //       A point to look at      (0, 0, 0)
            //       An "up" direction       <0, 1, 0>
            m_Device.Transform.View = Matrix.LookAtLH(
                new Vector3(0, 3, -10),
                new Vector3(0, 0, 0),
                new Vector3(0, 1, 0));

            // Projection Matrix:
            // Perspective transformation defined by:
            //       Field of view           Pi / 4
            //       Aspect ratio            1
            //       Near clipping plane     Z = 1
            //       Far clipping plane      Z = 10
            m_Device.Transform.Projection =
                Matrix.PerspectiveFovLH((float)(Math.PI / 4f), 1, 1, 100);
        }

        // Make the material.
        private void SetupMaterial()
        {
            m_Device.Material = new Material() 
                { Ambient = Color.Gray, Diffuse = Color.Gray };
        }

        // Make the lights.(We only use one.)
        private void SetupLights()
        {
            // Make a light.
            m_Device.Lights[0].Type = LightType.Directional;
            m_Device.Lights[0].Diffuse = Color.Green;
            m_Device.Lights[0].Ambient = Color.DarkGreen;
            m_Device.Lights[0].Direction = new Vector3(1, -1, 0);
            m_Device.Lights[0].Enabled = true;

            m_Device.Lights[1].Type = LightType.Directional;
            m_Device.Lights[1].Diffuse = Color.DarkRed;
            m_Device.Lights[1].Ambient = Color.DarkRed;
            m_Device.Lights[1].Direction = new Vector3(0, 1, 0);
            m_Device.Lights[1].Enabled = true;

            // Add some ambient light.
            m_Device.RenderState.Ambient = Color.FromArgb(255, 32, 32, 32);
        }
    }
}
